==> RSTouchscreenNG-1.3.0.zip: RSTouchscreenNG/projects/front-ui/src/lib/webcam.component.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, ViewChild, ElementRef, OnDestroy, HostBinding} from "@angular/core";
import {RepetierServer, RSUtils} from "binding";
import type {RepetierState, PrinterState, PrintersState, ConnectionState} from "binding";
import {SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  selector: "rs-webcam",
  template: `
        <div style="position: relative;height:100%" (rsClick)="toggleCam()" >
          <img class="reccam" src="/img/REC.png" *ngIf="active?.rec" title="<?php _('Capturing') ?>" style="position: absolute;width:21px;height:21px;top:5px;left:5px">
          <img #img [src]="url" [ngStyle]="style">
        </div>
    `
})
export class RSWebcamComponent implements OnDestroy {
  @HostBinding("style.position") styPosition = "relative";
  @HostBinding("style.height") styHeight = "100%";
  disposed = false;
  private wasDyn = false;
  url = "/img/nowebcam.png";
  @Input() dynamic = true;
  @ViewChild("img", {static: true}) img: ElementRef;
  printers$;
  active: PrinterState = undefined;
  style: any = {};
  lastOrient = -1;
  lastCam = -1;
  webcamId = 0;
  connection: ConnectionState;

  private subscription: SubscriptionLike;
  private connectionSubscription;

  constructor(public server: RepetierServer,
              private store: Store<RepetierState>,
              private elem: ElementRef) {
    this.printers$ = store.select("printers");
    this.subscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.active = p.printers[p.active];
        this.updateRotation();
      }
    });
    this.connectionSubscription = store.select("connection").subscribe(c => {
      this.connection = c;
    });
    this.updateImg();
  }

  updateRotation = (force: boolean = false): void => {
    if (!this.active || !this.active.config || !this.active.config.webcams || !this.img || !this.img.nativeElement.clientWidth) {
      if (!this.disposed) {
        setTimeout(this.updateRotation, 50);
      }
      return;
    }
    if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length) {
      const orient = this.active.config.webcams[this.webcamId].orientation;
      if (orient === this.lastOrient && !force) {
        return;
      }
      const w = this.img.nativeElement.clientWidth;
      const h = this.img.nativeElement.clientHeight;
      let trans = "rotate(" + orient + "deg)";
      if (orient === 90 || orient === 270) {
        trans = "scale(" + (h / w) + ") " + trans;
      }
      this.style = {
        "-webkit-transform": trans,
        "-ms-transform": trans,
        "transform": trans,
      };
      this.lastOrient = orient;
    } else {
      const orient = 0;
      // const w = this.img.nativeElement.clientWidth;
      // const h = this.img.nativeElement.clientHeight;
      const trans = "rotate(" + orient + "deg)";
      this.style = {
        "-webkit-transform": trans,
        "-ms-transform": trans,
        "transform": trans,
      };
      this.lastOrient = orient;
    }
  }

  updateUrl() {
    const lastUrl = this.url;
    // tslint:disable-next-line
    if (this.dynamic && this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 2) === 2)) {
      if (!this.wasDyn || this.webcamId !== this.lastCam) {
        this.url = "/printer/cammjpg/" + this.active.slug + "?cam=" + this.webcamId + "&rand=" + RSUtils.createAlphaID(10) + "&sess=" + encodeURIComponent(this.connection.session);
      }
      this.wasDyn = true;
      // tslint:disable-next-line
    } else if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 1) === 1)) {
      this.url = "/printer/camjpg/" + this.active.slug + "?cam=" + this.webcamId + "&rand=" + RSUtils.createAlphaID(10) + "&sess=" + encodeURIComponent(this.connection.session);
      this.wasDyn = false;
    } else {
      this.url = "/img/nowebcam.png";
      this.wasDyn = false;
    }
    this.lastCam = this.webcamId;
    if (lastUrl !== this.url) {
      this.updateRotation(true);
    }
  }

  toggleCam() {
    this.webcamId++;
    if (this.webcamId >= this.active.config.webcams.length) {
      this.webcamId = 0;
    }
    this.updateUrl();
  }

  ngOnDestroy() {
    this.url = "/img/empty.png";
    this.img.nativeElement.src = this.url; // prevent loading after removing bug
    this.disposed = true;
    this.subscription.unsubscribe();
    this.img.nativeElement.parentNode.textContent = "";
  }

  updateImg = () => {
    if (this.disposed) {
      return;
    }
    if (this.active && this.active.config && this.active.config.webcams[this.webcamId]) {
      this.updateUrl();
      setTimeout(this.updateImg, 1000 * this.active.config.webcams[this.webcamId].reloadInterval);
    } else {
      setTimeout(this.updateImg, 1000);
    }
  }
}

==> RSTouchscreenNG-1.4.16.zip: RSTouchscreenNG/projects/front-ui/src/lib/webcam.component.ts <==
/* (C) 2016-2023 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, ViewChild, ElementRef, OnDestroy, HostBinding} from "@angular/core";
import {RepetierServer, RSUtils} from "binding";
import type {RepetierState, PrinterState, PrintersState, ConnectionState} from "binding";
import {SubscriptionLike} from "rxjs";
import {Store} from "@ngrx/store";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-webcam",
  template: `
        <div style="position: relative;height:100%" (rsClick)="toggleCam()" >
          <img class="reccam" src="/img/REC.png" *ngIf="active?.rec" title="<?php _('Capturing') ?>" style="position: absolute;width:21px;height:21px;top:5px;left:5px">
          <img #img [src]="url" [ngStyle]="style">
        </div>
    `
})
export class RSWebcamComponent implements OnDestroy {
  @HostBinding("style.position") styPosition = "relative";
  @HostBinding("style.height") styHeight = "100%";
  disposed = false;
  private wasDyn = false;
  url = "/img/nowebcam.png";
  @Input() dynamic = true;
  @ViewChild("img", {static: true}) img: ElementRef;
  printers$;
  active: PrinterState = undefined;
  style: any = {};
  lastOrient = -1;
  lastCam = -1;
  webcamId = 0;
  connection: ConnectionState;

  private subscription: SubscriptionLike;
  private connectionSubscription;

  constructor(public server: RepetierServer,
              private store: Store<RepetierState>,
              private elem: ElementRef) {
    this.printers$ = store.select("printers");
    this.subscription = this.printers$.subscribe((p: PrintersState) => {
      if (p.active) {
        this.active = p.printers[p.active];
        this.updateRotation();
      }
    });
    this.connectionSubscription = store.select("connection").subscribe(c => {
      this.connection = c;
    });
    this.updateImg();
  }

  updateRotation = (force: boolean = false): void => {
    if (!this.active || !this.active.config || !this.active.config.webcams || !this.img || !this.img.nativeElement.clientWidth) {
      if (!this.disposed) {
        setTimeout(this.updateRotation, 50);
      }
      return;
    }
    if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length) {
      const orient = this.active.config.webcams[this.webcamId].orientation;
      if (orient === this.lastOrient && !force) {
        return;
      }
      const w = this.img.nativeElement.clientWidth;
      const h = this.img.nativeElement.clientHeight;
      let trans = "rotate(" + orient + "deg)";
      if (orient === 90 || orient === 270) {
        trans = "scale(" + (h / w) + ") " + trans;
      }
      this.style = {
        "-webkit-transform": trans,
        "-ms-transform": trans,
        "transform": trans,
      };
      this.lastOrient = orient;
    } else {
      const orient = 0;
      // const w = this.img.nativeElement.clientWidth;
      // const h = this.img.nativeElement.clientHeight;
      const trans = "rotate(" + orient + "deg)";
      this.style = {
        "-webkit-transform": trans,
        "-ms-transform": trans,
        "transform": trans,
      };
      this.lastOrient = orient;
    }
  }

  updateUrl() {
    const lastUrl = this.url;
    // eslint-disable-next-line
    if (this.dynamic && this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 2) === 2)) {
      if (!this.wasDyn || this.webcamId !== this.lastCam) {
        this.url = "/printer/cammjpg/" + this.active.slug + "?cam=" + this.webcamId + "&rand=" + RSUtils.createAlphaID(10) + "&sess=" + encodeURIComponent(this.connection.session);
      }
      this.wasDyn = true;
      // eslint-disable-next-line
    } else if (this.webcamId >= 0 && this.webcamId < this.active.config.webcams.length && ((this.active.config.webcams[this.webcamId].method & 1) === 1)) {
      this.url = "/printer/camjpg/" + this.active.slug + "?cam=" + this.webcamId + "&rand=" + RSUtils.createAlphaID(10) + "&sess=" + encodeURIComponent(this.connection.session);
      this.wasDyn = false;
    } else {
      this.url = "/img/nowebcam.png";
      this.wasDyn = false;
    }
    this.lastCam = this.webcamId;
    if (lastUrl !== this.url) {
      this.updateRotation(true);
    }
  }

  toggleCam() {
    this.webcamId++;
    if (this.webcamId >= this.active.config.webcams.length) {
      this.webcamId = 0;
    }
    this.updateUrl();
  }

  ngOnDestroy() {
    this.url = "/img/empty.png";
    this.img.nativeElement.src = this.url; // prevent loading after removing bug
    this.disposed = true;
    this.subscription.unsubscribe();
    this.img.nativeElement.parentNode.textContent = "";
  }

  updateImg = () => {
    if (this.disposed) {
      return;
    }
    if (this.active && this.active.config && this.active.config.webcams[this.webcamId]) {
      this.updateUrl();
      setTimeout(this.updateImg, 1000 * this.active.config.webcams[this.webcamId].reloadInterval);
    } else {
      setTimeout(this.updateImg, 1000);
    }
  }
}
